#include <stdlib.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <linux/fs.h>
#include <linux/unistd.h>
#include "diskio.h"

int read_sector(DISKDEV *dev, sector_t sect, unsigned char *buf, int nsect)
{
    long long result, off;
    ssize_t len;

    off = (long long)sect << dev->sector_shift;

    if ((result = lseek64(dev->fd, off, SEEK_SET)) < 0)
	return -1;

    len = read(dev->fd, buf, nsect * dev->sector_size);
    return len == nsect * dev->sector_size ? 0 : -1;
}

int write_sector(DISKDEV *dev, sector_t sect, unsigned char *buf, int nsect)
{
    long long result,off;
    ssize_t len;

    off = (long long)sect << dev->sector_shift; 

    if ((result = lseek64(dev->fd, off, SEEK_SET)) < 0)
	return -1;

    len = write(dev->fd, buf, nsect * dev->sector_size);
    return len == nsect * dev->sector_size ? 0 : -1;
}

int reread_partition(DISKDEV *dev)
{
    sync();
    sleep(2);
    return ioctl(dev->fd, BLKRRPART, 0);
}

static int dev_getinfo(DISKDEV *dev)
{
    long nsect;

    if (ioctl(dev->fd, BLKGETSIZE, &nsect) < 0)
	return -1;
    dev->nsect = nsect;

    /* TODO */
    dev->sector_size = DEFAULT_SECTOR_SIZE;
    dev->sector_shift = DEFAULT_SECTOR_SHIFT;

    return 0;
}

DISKDEV *dev_open(char *devname, int readonly)
{
    DISKDEV *dev;
    int fd;

    if ((dev = malloc(sizeof(DISKDEV))) == NULL)
	return NULL;

    if ((fd = open(devname, readonly ? O_RDONLY : O_RDWR)) < 0) {
	free(dev);
	return NULL;
    }

    dev->devname = devname;
    dev->fd = fd;
    dev->sector_size = dev->sector_shift = 0;
    dev->nsect = 0;

    if (dev_getinfo(dev) != 0) {
	dev_close(dev);
	return NULL;
    }

    return dev;
}

int dev_close(DISKDEV *dev)
{
    close(dev->fd);
    free(dev);

    return 0;
}
